#encoding: utf-8
from django.utils.translation import ugettext as _
from django.conf import settings

MODELS = getattr(settings, 'PLUGSHOP_MODELS', {})
FORMS = getattr(settings, 'PLUGSHOP_FORMS', {})
CONFIG = getattr(settings, 'PLUGSHOP_CONFIG', {})
OPTIONS = getattr(settings, 'PLUGSHOP_OPTIONS', {})
MESSAGES = getattr(settings, 'PLUGSHOP_MESSAGES', {})

PRODUCT_MODEL_DEFAULT = 'plugshop.models.product.Product'
PRODUCT_MODEL = MODELS.get('PRODUCT', PRODUCT_MODEL_DEFAULT)

CATEGORY_MODEL_DEFAULT = 'plugshop.models.category.Category'
CATEGORY_MODEL = MODELS.get('CATEGORY', CATEGORY_MODEL_DEFAULT)

ORDER_MODEL_DEFAULT = 'plugshop.models.order.Order'
ORDER_MODEL = MODELS.get('ORDER', ORDER_MODEL_DEFAULT)
                
ORDER_PRODUCTS_MODEL_DEFAULT = 'plugshop.models.order_products.OrderProducts'
ORDER_PRODUCTS_MODEL = MODELS.get('ORDER_PRODUCTS', 
                                    ORDER_PRODUCTS_MODEL_DEFAULT)

ORDER_FORM_DEFAULT = 'plugshop.forms.OrderForm'
ORDER_FORM = FORMS.get('ORDER', ORDER_FORM_DEFAULT)

REQUEST_NAMESPACE = CONFIG.get('REQUEST_NAMESPACE', 'cart')
SESSION_NAMESPACE = CONFIG.get('SESSION_NAMESPACE', 'cart')
URL_PREFIX = CONFIG.get('URL_PREFIX', '')

URL_SUCCESS = OPTIONS.get('URL_SUCCESS', '/')
STATUS_CHOICES = OPTIONS.get('STATUS_CHOICES', (
                            (1, _('Created')),
                            (2, _('Confirmed')),
                            (3, _('Denied')),
                            (4, _('Shipped')),
                            (5, _('Completed')),
                        ))
STATUS_CHOICES_START = STATUS_CHOICES[0][0]
STATUS_CHOICES_FINISH = STATUS_CHOICES[-1][0]

MESSAGE_SUCCESS = MESSAGES.get('SUCCESS', 'Order created')


# encoding: utf-8
from django import template
from django.utils.translation import ugettext as _

register = template.Library()

class HasProduct(template.Node):
    def __init__(self, parser, token):
        try:
            tag_name, product = token.split_contents()
        except ValueError:
            raise template.TemplateSyntaxError, "%r tag requires exactly one argument" % token.contents.split()[0]

        nodelist = parser.parse(('endplusghop_has_product',))
        parser.delete_first_token()
        
        self.product = parser.compile_filter(product)
        self.nodelist = nodelist

    def render(self, context, *args, **kwargs):
        product = self.product.resolve(context, True)
        return self.nodelist.render(context)


@register.tag
def plusghop_has_product(parser, token, *args, **kwargs):
    return HasProduct(parser, token)

@register.filter
def plugshop_currency(value):
    if value is None: return ""

    v = str(value)[::-1]
    return " ".join([v[i:i+3][::-1] for i in xrange(0, len(v), 3) ][::-1])

@register.inclusion_tag('plugshop/tags/action.html')
def plugshop_action(product, action, quantity=1):
    return {
        'product': product,
        'action': action,
        'quantity': quantity,
    }


import time
from django.utils import simplejson as json

from plugshop import settings
from plugshop.utils import serialize_model


class CartItem(object):

    def __init__(self, product, price=0, quantity=1):
        self.product = product
        self.price = price
        self.quantity = quantity

    def price_total(self):
        return self.price * self.quantity

class Cart(list):

    def __init__(self, request, name):
        super(Cart, self).__init__()
        self.request = request
        self.name = name
        
        for item, price, quantity in request.session.get(self.name, []):
            self.append(item, price, quantity)

    def __len__(self): 
        return sum(c.quantity for c in self)

    def _get_product(self, product):
        try:
            return filter(lambda x: x.product.pk == product.pk, self)[0]
        except IndexError:
            return None
            
    def has_product(self, product):
        return self._get_product(product) or False

    def save(self):
        self.request.session[self.name] = tuple(
                (item.product, item.price, item.quantity) for item in self)
    
    def append(self, product, price=0, quantity=1):
        item = self._get_product(product)
        if item:
            self[self.index(item)].quantity += quantity
        else:
            super(Cart, self).append(
                CartItem(product, price, quantity)
            )
    
    def remove(self, product, quantity=None):
        item = self._get_product(product)
        if item:
            if quantity:
                if item.quantity > quantity:
                    self[self.index(item)].quantity -= quantity
                else:
                    super(Cart, self).remove(item)
            else:
                super(Cart, self).remove(item)

    def empty(self):
        while len(self):
            self.pop()

    def price_total(self):
        return sum([p.price_total() for p in self])
        
    def serialize(self):
        data = {
            'products': self.get_products(),
            'price_total': self.price_total(),
            'goods_total': len(self)
        }
        return data

    def get_products(self):
        return [{
                'product': serialize_model(item.product),
                'price': item.price,
                'price_total': item.price_total(),
                'quantity': item.quantity
            } for item in self]
                


def get_cart(request):
    return getattr(request, settings.REQUEST_NAMESPACE, None)



from django import forms
from django.utils.translation import ugettext as _
from django.contrib.auth.models import User
from plugshop import settings
from plugshop.utils import load_class

PRODUCT_CLASS = load_class(settings.PRODUCT_MODEL)
ORDER_CLASS = load_class(settings.ORDER_MODEL)
ORDER_PRODUCTS_CLASS = load_class(settings.ORDER_PRODUCTS_MODEL)
NAME_ERROR = _('Name is required')
EMAIL_ERROR = _('Invalid email address')
EMAIL_ERROR_EXISTS = _("Email address '%s' already exits, must be unique")

class ProductForm(forms.Form):
    product = forms.ModelChoiceField(queryset=PRODUCT_CLASS.objects)
    quantity = forms.IntegerField(required=False)

class OrderForm(forms.ModelForm):

    class Meta:
        model = ORDER_CLASS
        exclude = (
            'number', 
            'status',
            'created_at',
            'updated_at',
            'delivered_at',
            'user',
            'products',
        )

    name = forms.CharField(required=True, error_messages={
                                            'required': NAME_ERROR
                                        })
    email = forms.EmailField(required=True, error_messages={
                                            'required': EMAIL_ERROR
                                        })

    def save(self, commit=True, **kwargs):
        cart = kwargs.get('cart')

        user, created = User.objects.get_or_create(
            email = self.cleaned_data.get('email')
        )
        if created:
            user.username = self.cleaned_data.get('email')
            user.email = self.cleaned_data.get('email')
            user.first_name = self.cleaned_data.get('first_name', '')
            user.last_name = self.cleaned_data.get('last_name', '')
            user.is_active = False
            user.save()
        
        model = super(OrderForm, self).save(commit=False)
        model.user = user

        if commit:
            model.save()
            for c in cart:
                ORDER_PRODUCTS_CLASS.objects.create(
                    product=c.product,
                    quantity=c.quantity,
                    order=model
                )
                
        return model

    def clean_name(self):
        name = self.cleaned_data.get('name').strip()
        if len(name):
            n = name.split()
            if len(n) > 1:
                self.cleaned_data['first_name'] = n[0]
                self.cleaned_data['last_name']  = n[1]
            else:
                self.cleaned_data['first_name'] = name
                self.cleaned_data['last_name'] = ''
        else:
            raise forms.ValidationError(NAME_ERROR)
        return name


import time
from plugshop import settings
from plugshop.cart import Cart

class CartMiddleware(object):

    def process_request(self, request):
        setattr(request, 
                settings.REQUEST_NAMESPACE, 
                Cart(request, settings.SESSION_NAMESPACE))

    def process_response(self, request, response):
        if hasattr(request, settings.REQUEST_NAMESPACE):
            cart = getattr(request, settings.REQUEST_NAMESPACE)
            cart.save()
        return response


import datetime
from django.db import models
from django.db.models.signals import pre_save
from django.dispatch import receiver
from django.utils.translation import ugettext as _

from plugshop import settings
from plugshop.utils import load_class, is_default_model

class OrderAbstract(models.Model):

    class Meta:
        abstract = True
        verbose_name = _('order')
        verbose_name_plural = _('orders')
    
    number = models.IntegerField(_('order number'), unique=True, 
                                editable=False)
    status = models.IntegerField(_('order status'), blank=False, 
                                choices=settings.STATUS_CHOICES, 
                                default=settings.STATUS_CHOICES_START)
    created_at = models.DateTimeField(_('creation date'), blank=False, 
                                        default=datetime.datetime.now)
    updated_at = models.DateTimeField(_('updated at'), blank=True, null=True)
    delivered_at = models.DateTimeField(_('delivery date'), blank=True, 
                                        null=True)
                                        
    def get_price(self):
        items = load_class(settings.ORDER_PRODUCTS_MODEL).objects.filter(
                                                                    order=self)
        return sum(item.quantity * item.product.price for item in items)
    
    def __unicode__(self):
        return str(self.pk)


if is_default_model('ORDER'):
    class Order(OrderAbstract):
        class Meta:
            verbose_name = _('order')
            verbose_name_plural = _('orders')
            app_label = 'plugshop'


#encoding: utf-8
import plugshop.utils as utils

from django.contrib import admin
from django.utils.translation import ugettext as _

from mptt.admin import MPTTModelAdmin

from plugshop import settings
from plugshop.utils import load_class
from plugshop.models import *
from plugshop.utils import is_default_model

class BaseProductAdmin(admin.ModelAdmin):
    #change_list_template = 'admin/product/change_list.html'
    inlines = ()
    search_fields = ('name',)
    prepopulated_fields = {
        'slug': ('name',) 
    }
    list_display = (
        'name',
        'slug',
        'price',
        #'category',
    )
    list_filter = (
        'category',
    )
    def changelist_view(self, request, extra_context=None):
        context = {}
        return super(BaseProductAdmin, self).changelist_view(request, 
                                                        extra_context=context)

if is_default_model('PRODUCT'):
    admin.site.register(load_class(settings.PRODUCT_MODEL), BaseProductAdmin)

class BaseCategoryAdmin(MPTTModelAdmin):
    change_list_template = 'admin/category/change_list.html'
    mptt_level_indent = 20
    prepopulated_fields = {
        'slug': ('name', )
    }
    list_display = (
        'name',
        'get_products',
        'slug',
    )

    def get_products(self, instance):
        return "<br/>".join(p.name for p in instance.products.all())
    get_products.allow_tags = True
    get_products.short_description = _('products')
    
if is_default_model('CATEGORY'):
    admin.site.register(load_class(settings.CATEGORY_MODEL), BaseCategoryAdmin)



class BaseOrderProductsInline(admin.TabularInline):
    model = load_class(settings.ORDER_PRODUCTS_MODEL)
    extra = 0

class BaseOrderAdmin(admin.ModelAdmin):
    date_hierarchy = 'created_at'
    inlines = (
        BaseOrderProductsInline,
    )
    search_fields = ('number', 'user',)
    list_display = (
        'number',
        'user',
        'status',
        'created_at',
        'updated_at',
        'delivered_at',
    )
    list_editable = (
        'status',
    )
    list_filter = (
        'status', 
        'created_at',
    )
if is_default_model('ORDER'):
    admin.site.register(load_class(settings.ORDER_MODEL), BaseOrderAdmin)



# encoding: utf-8
from django.db import models
from django.utils.translation import ugettext as _
from django.core.exceptions import ObjectDoesNotExist

from mptt.models import MPTTModel, TreeForeignKey
from mptt.managers import TreeManager

from plugshop import settings
from plugshop.utils import load_class, get_categories, is_default_model

class CategoryAbstractManager(TreeManager):
    def get_by_path(self, path):
        path_patterns = path.split('/')
        slug = path_patterns[-1]
        try:
            return self.get(slug=slug)
        except ObjectDoesNotExist:
            raise load_class(settings.CATEGORY_MODEL).DoesNotExist

class CategoryAbstract(MPTTModel):

    objects = CategoryAbstractManager()

    class Meta:
        abstract = True
        verbose_name = _('category')
        verbose_name_plural = _('categories')

    class MPTTMeta:
        ordering = ['pk', 'lft']

    parent = TreeForeignKey('self', null=True, blank=True)
    name = models.CharField(_('name'), blank=False, max_length=80)
    slug = models.SlugField(_('slug'), blank=False, unique=True)

    def __unicode__(self):
        return self.name
        
    def get_ancestor_list(self):
        categories = get_categories()
        return [n for n in categories 
                    if n.lft <= self.lft and 
                        n.rght >= self.rght and 
                            n.tree_id == self.tree_id]

    def get_path(self):
        ancestors = self.get_ancestor_list()
        return "/".join([a.slug for a in ancestors])

    @models.permalink
    def get_absolute_url(self):
        return ('plugshop-category', None, {'category_path': self.get_path() })



if is_default_model('CATEGORY'):
    class Category(CategoryAbstract):
        class Meta:
            verbose_name = _('category')
            verbose_name_plural = _('categories')
            app_label = 'plugshop'


from django.db import models
from django.utils.translation import ugettext as _
from plugshop.utils import is_default_model

class OrderProductsAbstract(models.Model):
    class Meta:
        abstract = True
        verbose_name = _('order product')
        verbose_name_plural = _('order product')

    quantity = models.PositiveIntegerField(_('quantity'), 
                                            blank=False, 
                                            null=False, 
                                            default=1)

if is_default_model('ORDER_PRODUCTS'):
    class OrderProducts(OrderProductsAbstract):
        class Meta:
            app_label = 'plugshop'
            verbose_name = _('order product')
            verbose_name_plural = _('order product')


# encoding: utf-8
from django.conf.urls import patterns, url
from django.utils.translation import ugettext as _

from plugshop import settings
from plugshop.views import *
from plugshop.exceptions import NoUrlFound

PREFIX = settings.URL_PREFIX

urlpatterns = patterns('plugshop.views',
    url(r"^$", ProductListView.as_view(), 
            name='plugshop-product-list'),
    url(r"^cart/$", CartView.as_view(), 
            name="plugshop-cart"),
    url(r"^order/$", OrderView.as_view(), 
            name="plugshop-order"),
    url(r"^(?P<category_path>[\-\/\w]+)/$", CategoryView.as_view(), 
            name='plugshop-category'),
    url(r"^(?P<category_path>[\-\/\w]+)/(?P<slug>[\-\/\w]+)$", 
            ProductView.as_view(), 
            name='plugshop-product'),
)

def get_url(name):
    patterns = filter(lambda x: x.name == name, urlpatterns)
    try:
        pattern = patterns[0]
    except IndexError:
        raise NoUrlFound(_("No url with name '%s'" % name))

    return pattern._regex
        


from django.db import models
from django.utils.translation import ugettext as _
from plugshop.utils import is_default_model, get_categories

class ProductAbstract(models.Model):
    class Meta:
        abstract = True
        verbose_name  = _('product')
        verbose_name_plural = _('products')

    name = models.CharField(_('name'), blank=False, max_length=200)
    slug = models.SlugField(_('slug'), blank=False, unique=True)
    price = models.PositiveIntegerField(_('price'), blank=False)

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        categories = get_categories()
        try:
            category = filter(lambda x: x.pk == self.category_id, categories)[0]
            category_path = category.get_path()
        except IndexError:
            category_path = "-"

        return ('plugshop-product', None, {
            'category_path': category_path,
            'slug': self.slug,
        })

if is_default_model('PRODUCT'):
    class Product(ProductAbstract):
        class Meta:
            app_label = 'plugshop'
            verbose_name  = _('product')
            verbose_name_plural = _('products')



import datetime
from django.utils.translation import ugettext as _
from django.db import models
from django.contrib.auth.models import User

from plugshop import settings
from plugshop.utils import load_class, get_categories

from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver

from plugshop.models.product import *
from plugshop.models.category import *
from plugshop.models.order import *
from plugshop.models.order_products import *
from mptt.fields import TreeForeignKey

PRODUCT_CLASS = load_class(settings.PRODUCT_MODEL)
CATEGORY_CLASS = load_class(settings.CATEGORY_MODEL)
ORDER_CLASS = load_class(settings.ORDER_MODEL)
ORDER_PRODUCTS_CLASS = load_class(settings.ORDER_PRODUCTS_MODEL)

PRODUCT_CLASS.add_to_class('category', TreeForeignKey(CATEGORY_CLASS,
                                        verbose_name=_('category'),
                                        related_name='products',
                                        blank=True,
                                        null=True))

ORDER_CLASS.add_to_class('products', models.ManyToManyField(PRODUCT_CLASS,
                                        through=ORDER_PRODUCTS_CLASS,
                                        related_name="products",
                                        verbose_name=_('products')))

ORDER_CLASS.add_to_class('user', models.ForeignKey(User,
                                        verbose_name=_('user')))

ORDER_PRODUCTS_CLASS.add_to_class('order', models.ForeignKey(ORDER_CLASS,
                                        verbose_name=_('order')))

ORDER_PRODUCTS_CLASS.add_to_class('product', models.ForeignKey(PRODUCT_CLASS, 
                                        verbose_name=_('product')))

@receiver(pre_save, sender=ORDER_CLASS)
def set_delivered(sender, instance, **kwargs):
    if instance.status == settings.STATUS_CHOICES_FINISH:
        instance.delivered_at = datetime.datetime.now()
    else:
        instance.delivered_at = None

@receiver(pre_save, sender=ORDER_CLASS)
def set_updated(sender, instance, **kwargs):
    instance.updated_at = datetime.datetime.now()

@receiver(pre_save, sender=ORDER_CLASS)
def generate_number(sender, instance, **kwargs):
    if instance.id is None:
        today = datetime.datetime.now()
        today_orders = ORDER_CLASS.objects.filter(
                            created_at__year=today.year,
                            created_at__month=today.month,
                            created_at__day=today.day
                        )
        num = int("%s%s" % (today.strftime("%y%m%d"), len(today_orders) + 1))
        instance.number = num

post_save.connect(get_categories, sender=CATEGORY_CLASS)


from plugshop import settings
from django.core import exceptions
from django.utils.importlib import import_module
from django.db.models import get_model
from django.core.cache import cache

def load_class(path):
    module_path, class_name = path.rsplit('.', 1)
    module = import_module(module_path)
    cl = getattr(module, class_name)
    return cl

def is_default_model(name):
    model = getattr(settings, "%s_MODEL" % name, None)
    default_model = getattr(settings, "%s_MODEL_DEFAULT" % name, None)

    if model is not None and default_model is not None:
        return model == default_model
    else:
        return False

def serialize_model(instance):
    data = {}
    for field in instance._meta.fields:
        data[field.name] = field.value_to_string(instance)
    return data


def serialize_queryset(queryset):
    return [serialize_model(item) for item in queryset]

def get_categories(*args, **kwargs):
    categories = cache.get('plugshop_categories')
    if categories is None:
        categories = load_class(settings.CATEGORY_MODEL).objects.all()
        cache.set('plugshop_categories', categories)
    return categories





import os
from setuptools import setup, find_packages
import plugshop 

setup(
    author = "Eugene Cheltsov",
    author_email = "chill.icp@gmail.com",
    classifiers = [
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
    description = ("A small shop for Django Framework"),
    download_url='',
    include_package_data = True,
    install_requires = (
        'Django>=1.3.1',
        'django-mptt>=0.5.2',
    ),
    keywords = "plugshop",
    long_description = open(os.path.join(os.path.dirname(__file__), 'README.md')).read(),
    license = "BSD",
    name = "django-plugshop",
    packages = find_packages(exclude=['testshop']),
    platforms = ['OS Independent'],
    url = "https://github.com/ChillyBwoy/django-plugshop",
    version = plugshop.__version__,
    zip_safe = False
)


# encoding: utf-8
from django.conf import settings as django_settings
from django.db.models.query import QuerySet
from django.views.decorators.csrf import csrf_protect, csrf_exempt
from django.shortcuts import get_object_or_404, redirect
from django.http import Http404, HttpResponse
from django.utils.translation import ugettext as _
from django.utils.decorators import method_decorator
from django.core import serializers
from django.views.generic import View, TemplateView, ListView, DetailView,\
CreateView, FormView
from django.views.generic.base import TemplateResponseMixin
from django.utils import simplejson as json
from django.contrib.auth.models import User
from django.contrib import messages

from django.utils.html import strip_tags
from django.template.loader import render_to_string
from django.core.mail import EmailMessage, mail_managers, \
mail_admins

from plugshop.utils import serialize_queryset
from plugshop import settings
from plugshop.utils import load_class, serialize_model, serialize_queryset
from plugshop.forms import *
from plugshop.cart import get_cart

PRODUCT_CLASS = load_class(settings.PRODUCT_MODEL)
CATEGORY_CLASS = load_class(settings.CATEGORY_MODEL)
ORDER_CLASS = load_class(settings.ORDER_MODEL)
ORDER_PRODUCTS_CLASS = load_class(settings.ORDER_PRODUCTS_MODEL)

ORDER_FORM_CLASS = load_class(settings.ORDER_FORM)

class ProductListView(ListView):
    context_object_name = 'products'
    template_name = 'plugshop/product_list.html'
    model = PRODUCT_CLASS

    def get_context_data(self, **kwargs):
        context = super(ProductListView, self).get_context_data(**kwargs)
        categories = CATEGORY_CLASS.objects.all()
        context.update(
            categories = categories
        )
        return context

class ProductView(DetailView):
    model = PRODUCT_CLASS
    context_object_name = 'product'
    template_name = 'plugshop/product_detail.html'
    
    def get_object(self, *args, **kwargs):
        slug = self.kwargs.get('slug', None)
        return get_object_or_404(PRODUCT_CLASS, slug=slug)

    def get_context_data(self, *args, **kwargs):
        context = super(ProductView, self).get_context_data(**kwargs)
        product = context.get('product')
        context.update(
            category=product.category
        )
        return context


class CategoryView(DetailView):
    model = CATEGORY_CLASS
    context_object_name = 'category'
    template_name = 'plugshop/product_list.html'

    def get_object(self, *args, **kwargs):
        path = self.kwargs.get('category_path', None)
        try:
            return CATEGORY_CLASS.objects.get_by_path(path)
        except CATEGORY_CLASS.DoesNotExist:
            raise Http404

    def get_context_data(self, **kwargs):
        context = super(CategoryView, self).get_context_data(**kwargs)
        category = context.get('category')
        
        category_list = [category] + list(category.get_children())
        products = PRODUCT_CLASS.objects.filter(category__in=category_list)

        categories = CATEGORY_CLASS.objects.all()
        context.update(
            products = products,
            categories = categories
        )
        return context

class CartView(TemplateResponseMixin, View):
    template_name = 'plugshop/cart.html'

    def extend_context(self, context):
        return context
        
    def extend_context_ajax(self, context):
        return context
    
    def get(self, request, **kwargs):
        cart = request.cart
        context = {}

        if request.is_ajax():
            context['cart'] = cart.serialize()
            return HttpResponse(json.dumps(context), 
                                content_type='application/json', **kwargs)
        else:
            context['form'] = ORDER_FORM_CLASS()
            if len(cart) == 0:
                return redirect('plugshop-product-list')
            else:
                return self.render_to_response(context)


    def post(self, request, **kwargs):
        action = request.POST.get('_action', None)
        cart = get_cart(request)
        
        if action == 'remove_all':
            cart.empty()
        else:
            form = ProductForm(request.POST)

            if form.is_valid():
                product = form.cleaned_data.get('product')
                quantity = form.cleaned_data.get('quantity', 1)
                
                if action == 'add':
                    cart.append(product, int(product.price), quantity)

                elif action == 'remove':
                    cart.remove(product, quantity)

                elif action == 'remove_product':
                    cart.remove(product)

                else:
                    raise Http404
        cart.save()
        if request.is_ajax():
            return HttpResponse(json.dumps({'cart': cart.serialize()}), 
                                content_type='application/json', **kwargs)
        else:
            return redirect('plugshop-cart')



class OrderSuccessView(DetailView):
    template_name = 'plugshop/order_success.html'


class OrderView(FormView):
    template_name = 'plugshop/order_form.html'
    form_class = ORDER_FORM_CLASS
    success_url = settings.URL_SUCCESS
    
    def get_context_data(self, **kwargs):
        context = super(OrderView, self).get_context_data(**kwargs)
        return context
        
    def get_form(self, form_class):
        return form_class(**self.get_form_kwargs())

    def get_initial(self, *args, **kwargs):
        cart = get_cart(self.request)
        user = self.request.user
        initial = {}
        if user.is_authenticated():
            initial.update({
                'name': "%s %s" % (user.first_name, user.last_name),
                'email': user.email
            })

        return initial

    def get_form_kwargs(self):
        cart = get_cart(self.request)
        form_kwargs = super(OrderView, self).get_form_kwargs()
        return form_kwargs
    
    def notify_managers(self, order):
        cart = get_cart(self.request)
        msg = render_to_string('plugshop/email/order_admin.html', {
            'cart': cart,
            'order': order,
            'total': cart.price_total(),
        })
        mail_managers(_('New Order'), '', html_message=msg)
        
    def notify_customer(self, order):
        cart = get_cart(self.request)
        
        msg = render_to_string('plugshop/email/order_user.html', {
            'cart': cart,
            'order': order,
            'total': cart.price_total(),
        })
        mail = EmailMessage(_('New Order'), msg, django_settings.SERVER_EMAIL, [order.user.email])
        mail.content_subtype = 'html'
        mail.send()

    def form_valid(self, form):
        cart = get_cart(self.request)
        order = form.save(cart=cart)
        
        self.notify_managers(order)
        self.notify_customer(order)

        messages.info(self.request, settings.MESSAGE_SUCCESS)
        cart.empty()
        return super(OrderView, self).form_valid(form)

    def form_invalid(self, form):
        return self.render_to_response(self.get_context_data(form=form))